@@ -76,13 +76,13 @@ def flaws(ctx):
 @click.pass_context
 @progress_bar
 def list_flaws(ctx, **params):
-    # TODO: handle pagination
     # TODO: handle output
     session = OSIDBService.create_session()
 
     params = multivalue_params_to_csv(params)
-    data = session.flaws.retrieve_list(**params).results
-    return cprint(data, ctx=ctx)
+    return cprint(
+        list(session.flaws.retrieve_list_iterator_async(max_results=5000, **params)), ctx=ctx
+    )
 
 
 @flaws.command(name="get")
@@ -558,8 +558,9 @@ def list_affects(ctx, **params):
     session = OSIDBService.create_session()
 
     params = multivalue_params_to_csv(params)
-    data = session.affects.retrieve_list(**params).results
-    return cprint(data, ctx=ctx)
+    return cprint(
+        list(session.affects.retrieve_list_iterator_async(max_results=5000, **params)), ctx=ctx
+    )
 
 
 @affects.command(name="get")
@@ -730,8 +731,9 @@ def list_trackers(ctx, **params):
     session = OSIDBService.create_session()
 
     params = multivalue_params_to_csv(params)
-    data = session.trackers.retrieve_list(**params).results
-    return cprint(data, ctx=ctx)
+    return cprint(
+        list(session.trackers.retrieve_list_iterator_async(max_results=5000, **params)), ctx=ctx
+    )
 
 
 @trackers.command(name="get")
