@@ -1,3 +1,7 @@
+"""
+    Gather up all of the messy 'presentation' logic into one place
+
+"""
 import enum
 import json
 import logging
@@ -261,20 +265,25 @@ def generate_affects(
     ctx, result_tree, exclude_components, flaw_operation, format="text", no_wrap=False
 ):
     search_component_name = ctx.params["component_name"]
+    affects = []
+
     for pv in result_tree.keys():
         component_names = set()
         for ps in result_tree[pv].keys():
             for component_name in result_tree[pv][ps].keys():
                 for nvr in result_tree[pv][ps][component_name].keys():
-                    if len(result_tree[pv][ps][component_name][nvr]["sources"]) > 0:
+                    if result_tree[pv][ps][component_name][nvr]["sources"]:
                         source_names = [
-                            source["name"]
+                            source.name
                             for source in result_tree[pv][ps][component_name][nvr]["sources"]
-                            if source.get("namespace") == "REDHAT"
+                            if source.namespace == "REDHAT"
                         ]
                         component_names.update(source_names)
                     else:
-                        if result_tree[pv][ps][component_name][nvr]["namespace"] == "REDHAT":
+                        if ctx.params["no_upstream_affects"]:
+                            if result_tree[pv][ps][component_name][nvr]["namespace"] == "REDHAT":
+                                component_names.add(component_name)
+                        else:
                             component_names.add(component_name)
 
         # we should only show component name if both {component name} and {component name-container} exists # noqa
@@ -292,14 +301,13 @@ def generate_affects(
                         no_wrap=no_wrap,
                     )
         else:
-            affects = []
             for cn in component_names:
                 # ensure {component name} is not in profile exclude components enum
                 if not any([re.search(match, cn) for match in exclude_components]):
                     affects.append(
                         {"product_version": pv, "component_name": cn, "operation": flaw_operation}
                     )
-            return affects
+    return affects
 
 
 def text_output_products_contain_component(
@@ -325,9 +333,6 @@ def text_output_products_contain_component(
     if "results" in output and output["count"] > 0:
         console.highlighter = None
 
-        # order at the end
-        # ordered_results = sorted(output["results"], key=lambda d: d["product_stream"])
-
         # first flatten the tree
         normalised_results = generate_normalised_results(
             output,
@@ -345,11 +350,13 @@ def text_output_products_contain_component(
             console.highlighter = None
 
             flaw_mode = ctx.params["flaw_mode"]
-            flaw_operation = "dry_run"
+            flaw_operation = "new"
             if flaw_mode == "add":
                 flaw_operation = "new"
             if flaw_mode == "update":
                 flaw_operation = "update"
+            if flaw_mode == "dry_run":
+                flaw_operation = "dry_run"
 
             generate_affects(ctx, result_tree, exclude_components, flaw_operation, no_wrap=False)
 
@@ -365,9 +372,9 @@ def text_output_products_contain_component(
                         and f"{search_component_name}-container" in component_names
                     ):
                         component_names.remove(f"{search_component_name}-container")
-                    for cn in component_names:
+                    for cn in sorted(component_names):
                         # highlight search term
-                        dep_name = re.sub(cn, f"[b]{cn}[/b]", cn)
+                        dep_name = re.sub(re.escape(cn), f"[b]{cn}[/b]", cn)
                         dep = f"[grey93]{dep_name}[/grey93]"
                         console.print(
                             Text(pv, style="magenta b u"),
@@ -377,14 +384,14 @@ def text_output_products_contain_component(
             if ctx.obj["VERBOSE"] == 1:  # product_stream X nvr x related_url
                 for pv in result_tree.keys():
                     for ps in result_tree[pv].keys():
-                        for cn in result_tree[pv][ps].keys():
+                        for cn in sorted(result_tree[pv][ps].keys()):
                             # select the latest nvr (from sorted list)
                             nvr = list(result_tree[pv][ps][cn].keys())[-1]
                             # highlight search term
                             dep_name = nvr
                             try:
                                 dep_name = re.sub(
-                                    search_component_name,
+                                    re.escape(search_component_name),
                                     f"[b]{search_component_name}[/b]",
                                     nvr,
                                 )
@@ -395,55 +402,66 @@ def text_output_products_contain_component(
                             try:
                                 if result_tree[pv][ps][cn][nvr]["related_url"]:
                                     related_url = re.sub(
-                                        search_component_name,
+                                        re.escape(search_component_name),
                                         f"[b]{search_component_name}[/b]",
                                         result_tree[pv][ps][cn][nvr]["related_url"],
                                     )
                             except re.error:
                                 pass
-                            upstream_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["upstreams"]
-                                    ]
+                            if result_tree[pv][ps][cn][nvr]["upstreams"]:
+                                upstream_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "upstreams"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            )
-                            if len(upstream_component_names) > 0:
-                                upstream_component_name = (
-                                    f"[cyan]{upstream_component_names[0]}[/cyan]"
+                                if len(upstream_component_names) > 0:
+                                    upstream_component_name = (
+                                        f"[cyan]{upstream_component_names[0]}[/cyan]"
+                                    )
+                                    if len(upstream_component_names) > 1:
+                                        upstream_component_name = f"[cyan]{upstream_component_names[0]} and {len(upstream_component_names) - 1} more[/cyan]"  # noqa
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        upstream_component_name,
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if result_tree[pv][ps][cn][nvr]["sources"]:
+                                source_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "sources"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                                if len(upstream_component_names) > 1:
-                                    upstream_component_name = f"[cyan]{upstream_component_names[0]} and {len(upstream_component_names) - 1} more[/cyan]"  # noqa
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    upstream_component_name,
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            source_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["sources"]
-                                    ]
-                                )
-                            )
-                            if len(source_component_names) > 0:
-                                source_component_name = f"[red]{source_component_names[0]}[/red]"
-                                if len(source_component_names) > 1:
-                                    source_component_name = f"[red]{source_component_names[0]} and {len(source_component_names) - 1} more[/red]"  # noqa
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    source_component_name,
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            if (
-                                len(result_tree[pv][ps][cn][nvr]["upstreams"]) == 0
-                                and len(result_tree[pv][ps][cn][nvr]["sources"]) == 0
+                                if len(source_component_names) > 0:
+                                    source_component_name = (
+                                        f"[red]{source_component_names[0]}[/red]"
+                                    )
+                                    if len(source_component_names) > 1:
+                                        source_component_name = f"[red]{source_component_names[0]} and {len(source_component_names) - 1} more[/red]"  # noqa
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        source_component_name,
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if not (result_tree[pv][ps][cn][nvr]["upstreams"]) and not (
+                                result_tree[pv][ps][cn][nvr]["sources"]
                             ):
                                 console.print(
                                     Text(ps, style="magenta b u"),
@@ -454,13 +472,13 @@ def text_output_products_contain_component(
             if ctx.obj["VERBOSE"] == 2:  # product_stream X nvr x related_url x build_source_url
                 for pv in result_tree.keys():
                     for ps in result_tree[pv].keys():
-                        for cn in result_tree[pv][ps].keys():
+                        for cn in sorted(result_tree[pv][ps].keys()):
                             nvr = list(result_tree[pv][ps][cn].keys())[-1]
                             # highlight search term
                             dep_name = nvr
                             try:
                                 dep_name = re.sub(
-                                    search_component_name,
+                                    re.escape(search_component_name),
                                     f"[b]{search_component_name}[/b]",
                                     nvr,
                                 )
@@ -471,7 +489,7 @@ def text_output_products_contain_component(
                             try:
                                 if result_tree[pv][ps][cn][nvr]["related_url"]:
                                     related_url = re.sub(
-                                        search_component_name,
+                                        re.escape(search_component_name),
                                         f"[b]{search_component_name}[/b]",
                                         result_tree[pv][ps][cn][nvr]["related_url"],
                                     )
@@ -480,51 +498,62 @@ def text_output_products_contain_component(
                             build_source_url = ""
                             if result_tree[pv][ps][cn][nvr]["build_source_url"]:
                                 build_source_url = result_tree[pv][ps][cn][nvr]["build_source_url"]
-                            upstream_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["upstreams"]
-                                    ]
+                            if result_tree[pv][ps][cn][nvr]["upstreams"]:
+                                upstream_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "upstreams"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            )
-                            if len(upstream_component_names) > 0:
-                                upstream_component_name = (
-                                    f"[cyan]{upstream_component_names[0]}[/cyan]"
+                                if len(upstream_component_names) > 0:
+                                    upstream_component_name = (
+                                        f"[cyan]{upstream_component_names[0]}[/cyan]"
+                                    )
+                                    if len(upstream_component_names) > 1:
+                                        upstream_component_name = f"[cyan]{upstream_component_names[0]} and {len(upstream_component_names) - 1} more[/cyan]"  # noqa
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        upstream_component_name,
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if result_tree[pv][ps][cn][nvr]["sources"]:
+                                source_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "sources"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                                if len(upstream_component_names) > 1:
-                                    upstream_component_name = f"[cyan]{upstream_component_names[0]} and {len(upstream_component_names) - 1} more[/cyan]"  # noqa
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    upstream_component_name,
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            source_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["sources"]
-                                    ]
-                                )
-                            )
-                            if len(source_component_names) > 0:
-                                source_component_name = f"[red]{source_component_names[0]}[/red]"
-                                if len(source_component_names) > 1:
-                                    source_component_name = f"[red]{source_component_names[0]} and {len(source_component_names) - 1} more[/red]"  # noqa
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    source_component_name,
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            if (
-                                len(result_tree[pv][ps][cn][nvr]["upstreams"]) == 0
-                                and len(result_tree[pv][ps][cn][nvr]["sources"]) == 0
+                                if len(source_component_names) > 0:
+                                    source_component_name = (
+                                        f"[red]{source_component_names[0]}[/red]"
+                                    )
+                                    if len(source_component_names) > 1:
+                                        source_component_name = f"[red]{source_component_names[0]} and {len(source_component_names) - 1} more[/red]"  # noqa
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        source_component_name,
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if not (result_tree[pv][ps][cn][nvr]["upstreams"]) and not (
+                                result_tree[pv][ps][cn][nvr]["sources"]
                             ):
                                 console.print(
                                     Text(ps, style="magenta b u"),
@@ -537,14 +566,14 @@ def text_output_products_contain_component(
             ):  # product_stream X nvr (full source/upstreams) x related_url x build_source_url
                 for pv in result_tree.keys():
                     for ps in result_tree[pv].keys():
-                        for cn in result_tree[pv][ps].keys():
+                        for cn in sorted(result_tree[pv][ps].keys()):
                             # select the latest nvr (from sorted list)
                             nvr = list(result_tree[pv][ps][cn].keys())[-1]
                             # highlight search term
                             dep_name = nvr
                             try:
                                 dep_name = re.sub(
-                                    search_component_name,
+                                    re.escape(search_component_name),
                                     f"[b]{search_component_name}[/b]",
                                     nvr,
                                 )
@@ -555,7 +584,7 @@ def text_output_products_contain_component(
                             try:
                                 if result_tree[pv][ps][cn][nvr]["related_url"]:
                                     related_url = re.sub(
-                                        search_component_name,
+                                        re.escape(search_component_name),
                                         f"[b]{search_component_name}[/b]",
                                         result_tree[pv][ps][cn][nvr]["related_url"],
                                     )
@@ -564,43 +593,52 @@ def text_output_products_contain_component(
                             build_source_url = ""
                             if result_tree[pv][ps][cn][nvr]["build_source_url"]:
                                 build_source_url = result_tree[pv][ps][cn][nvr]["build_source_url"]
-                            upstream_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["upstreams"]
-                                    ]
+                            if result_tree[pv][ps][cn][nvr]["upstreams"]:
+                                upstream_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "upstreams"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            )
-                            for upstream_name in upstream_component_names:
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    f"[cyan]{upstream_name}[/cyan]",
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
+                                for upstream_name in upstream_component_names:
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        f"[cyan]{upstream_name}[/cyan]",
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if result_tree[pv][ps][cn][nvr]["sources"]:
+                                source_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.name
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "sources"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            source_component_names = list(
-                                set(
-                                    [
-                                        source["name"]
-                                        for source in result_tree[pv][ps][cn][nvr]["sources"]
-                                    ]
-                                )
-                            )
-                            for source_name in source_component_names:
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    f"[light_blue]{source_name}[/light_blue]",
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            if (
-                                not result_tree[pv][ps][cn][nvr]["upstreams"]
-                                and not result_tree[pv][ps][cn][nvr]["sources"]
+                                for source_name in source_component_names:
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        f"[light_blue]{source_name}[/light_blue]",
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if not (result_tree[pv][ps][cn][nvr]["upstreams"]) and not (
+                                result_tree[pv][ps][cn][nvr]["sources"]
                             ):
                                 console.print(
                                     Text(ps, style="magenta b u"),
@@ -614,14 +652,14 @@ def text_output_products_contain_component(
             ):  # product_stream X nvr (full source/upstreams) x related_url x build_source_url
                 for pv in result_tree.keys():
                     for ps in result_tree[pv].keys():
-                        for cn in result_tree[pv][ps].keys():
+                        for cn in sorted(result_tree[pv][ps].keys()):
                             # select the latest nvr (from sorted list)
                             nvr = list(result_tree[pv][ps][cn].keys())[-1]
                             # highlight search term
                             dep_name = nvr
                             try:
                                 dep_name = re.sub(
-                                    search_component_name,
+                                    re.escape(search_component_name),
                                     f"[b]{search_component_name}[/b]",
                                     nvr,
                                 )
@@ -632,7 +670,7 @@ def text_output_products_contain_component(
                             try:
                                 if result_tree[pv][ps][cn][nvr]["related_url"]:
                                     related_url = re.sub(
-                                        search_component_name,
+                                        re.escape(search_component_name),
                                         f"[b]{search_component_name}[/b]",
                                         result_tree[pv][ps][cn][nvr]["related_url"],
                                     )
@@ -641,43 +679,52 @@ def text_output_products_contain_component(
                             build_source_url = ""
                             if result_tree[pv][ps][cn][nvr]["build_source_url"]:
                                 build_source_url = result_tree[pv][ps][cn][nvr]["build_source_url"]
-                            upstream_component_names = list(
-                                set(
-                                    [
-                                        source["nvr"]
-                                        for source in result_tree[pv][ps][cn][nvr]["upstreams"]
-                                    ]
+                            if result_tree[pv][ps][cn][nvr]["upstreams"]:
+                                upstream_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.nvr
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "upstreams"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            )
-                            for upstream_name in upstream_component_names:
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    f"[cyan]{upstream_name}[/cyan]",
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
+                                for upstream_name in upstream_component_names:
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        f"[cyan]{upstream_name}[/cyan]",
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if result_tree[pv][ps][cn][nvr]["sources"]:
+                                source_component_names = sorted(
+                                    list(
+                                        set(
+                                            [
+                                                source.nvr
+                                                for source in result_tree[pv][ps][cn][nvr][
+                                                    "sources"
+                                                ]
+                                            ]
+                                        )
+                                    )
                                 )
-                            source_component_names = list(
-                                set(
-                                    [
-                                        source["nvr"]
-                                        for source in result_tree[pv][ps][cn][nvr]["sources"]
-                                    ]
-                                )
-                            )
-                            for source_name in source_component_names:
-                                console.print(
-                                    Text(ps, style="magenta b u"),
-                                    f"[light_blue]{source_name}[/light_blue]",
-                                    dep,
-                                    f"([grey]{related_url}[/grey])",
-                                    f"([grey]{build_source_url}[/grey])",
-                                    no_wrap=no_wrap,
-                                )
-                            if (
-                                not result_tree[pv][ps][cn][nvr]["upstreams"]
-                                and not result_tree[pv][ps][cn][nvr]["sources"]
+                                for source_name in source_component_names:
+                                    console.print(
+                                        Text(ps, style="magenta b u"),
+                                        f"[light_blue]{source_name}[/light_blue]",
+                                        dep,
+                                        f"([grey]{related_url}[/grey])",
+                                        f"([grey]{build_source_url}[/grey])",
+                                        no_wrap=no_wrap,
+                                    )
+                            if not (result_tree[pv][ps][cn][nvr]["upstreams"]) and not (
+                                result_tree[pv][ps][cn][nvr]["sources"]
                             ):
                                 console.print(
                                     Text(ps, style="magenta b u"),
@@ -1008,6 +1055,7 @@ def text_output_list(ctx, output, format, exclude_components, no_wrap=False):
         if "cve_id" in output["results"][0]:
             for row in output["results"]:
                 console.print(
+                    row["cve_id"],
                     row["title"],
                     row["state"],
                     row["impact"],
