@@ -2,12 +2,12 @@
     read only queries
 
 """
-import concurrent
 import logging
+import multiprocessing
 import re
+from functools import partial
 from typing import Any, Dict, List
 
-import requests
 from component_registry_bindings.bindings.python_client.models import Component
 
 from griffon import (
@@ -37,7 +37,7 @@ class product_stream_summary:
         self.strict_name_search = self.params.get("strict_name_search", None)
         self.all = self.params.get("all", None)
 
-    def execute(self) -> List[Dict[str, Any]]:
+    def execute(self, status=None) -> List[Dict[str, Any]]:
         cond = {}
 
         if self.ofuri:
@@ -109,7 +109,7 @@ class products_versions_affected_by_specific_cve_query:
         self.component_type = self.params.get("component_type")
         self.namespace = self.params.get("namespace")
 
-    def execute(self) -> dict:
+    def execute(self, status=None) -> dict:
         cond = {}
         if self.affectedness:
             cond["affectedness"] = self.affectedness
@@ -129,30 +129,19 @@ class products_versions_affected_by_specific_cve_query:
         results = list()
         product_versions = set()
         product_streams = set()
-        with concurrent.futures.ThreadPoolExecutor() as executor:
-            futures = []
-            for affect in affects:
-                futures.append(
-                    executor.submit(
-                        self.corgi_session.components.retrieve_list,
-                        name=affect.ps_component,
-                        latest_components_by_streams=True,
-                        include_fields="product_streams.name,product_versions.name",
-                    )
-                )
-            for future in concurrent.futures.as_completed(futures):
-                try:
-                    for c in future.result().results:
-                        results.append(c.to_dict())
-                except Exception as exc:
-                    logger.error("%r generated an exception: %s" % (future, exc))
-                    exit(0)
-
-            for c in results:
-                for ps in c["product_streams"]:
-                    product_streams.add(ps["name"])
-                for pv in c["product_versions"]:
-                    product_versions.add(ps["name"])
+        for affect in affects:
+            components = self.corgi_session.components.retrieve_list_iterator_async(
+                name=affect.ps_component,
+                latest_components_by_streams="True",
+                include_fields="product_streams.name,product_versions.name",
+            )
+            for c in components:
+                results.append(c.to_dict())
+        for c in results:
+            for ps in c["product_streams"]:
+                product_streams.add(ps["name"])
+            for pv in c["product_versions"]:
+                product_versions.add(ps["name"])
         return {
             "link": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",
             "cve_id": flaw.cve_id,
@@ -187,6 +176,7 @@ class products_containing_specific_component_query:
         "search_redhat",
         "no_community",
         "no_middleware",
+        "no_upstream_affects",
         "include_inactive_product_streams",
         "include_product_stream_excluded_components",
         "output_type_filter",
@@ -197,36 +187,46 @@ class products_containing_specific_component_query:
         self.params = params
         self.purl = self.params.get("purl")
 
-    def execute(self) -> dict:
+    def execute(self, status=None) -> dict:
         c = self.corgi_session.components.retrieve_list(
             purl=self.purl,
         )
         return c["product_streams"]
 
 
-def async_retrieve_components(corgi_session, params, components_initial, component_cnt):
-    components = list()
-    if component_cnt < 120:
-        components.extend(components_initial.results)
-    elif component_cnt > 120:
-        components.extend(components_initial.results)
-        with concurrent.futures.ThreadPoolExecutor() as executor:
-            futures = []
-            for batch in range(120, 1200, 120):
-                futures.append(
-                    executor.submit(
-                        corgi_session.components.retrieve_list,
-                        **params,
-                        offset=batch,
-                        limit=120,  # noqa
-                    )
-                )
-            for future in concurrent.futures.as_completed(futures):
-                try:
-                    components.extend(future.result().results)
-                except Exception as exc:
-                    logger.warning("%r generated an exception: %s" % (future, exc))
-    return components
+def async_retrieve_sources(self, purl):
+    params = {
+        "limit": 300,
+        "root_components": "True",
+        "released_components": "True",
+        "provides": purl,
+        "include_fields": "type,nvr,purl,name,namespace,download_url,related_url",
+    }
+    # TODO: remove cnt and max_result and cnt > 10000 on next stage-> prod deployment
+    cnt = self.components.count(**params)
+    if cnt > 10000:
+        return list(self.components.retrieve_list_iterator_async(max_results=5000, **params))
+    return list(self.components.retrieve_list_iterator_async(max_results=10000, **params))
+
+
+def async_retrieve_upstreams(self, purl):
+    params = {
+        "limit": 300,
+        "root_components": "True",
+        "upstreams": purl,
+        "include_fields": "type,nvr,purl,name,namespace,download_url,related_url",
+    }
+    # TODO: remove max_result on next stage-> prod deployment
+    return list(self.components.retrieve_list_iterator_async(max_results=10000, **params))
+
+
+def process_component(session, c):
+    """perform any neccessary sub retrievals."""
+    # we only need to resolve on REDHAT namespace components (for now)
+    if c.namespace == "REDHAT":
+        c.sources = async_retrieve_sources(session, c.purl)
+        c.upstreams = async_retrieve_upstreams(session, c.purl)
+    return c
 
 
 class products_containing_component_query:
@@ -251,6 +251,7 @@ class products_containing_component_query:
         "filter_rh_naming",
         "no_community",
         "no_middleware",
+        "no_upstream_affects",
         "include_inactive_product_streams",
         "include_product_stream_excluded_components",
         "output_type_filter",
@@ -273,13 +274,16 @@ class products_containing_component_query:
         self.search_upstreams = self.params.get("search_upstreams")
         self.filter_rh_naming = self.params.get("filter_rh_naming")
         self.no_community = self.params.get("no_community")
+        if not self.no_community:
+            self.community_session = CommunityComponentService.create_session()
 
-    def execute(self) -> List[Dict[str, Any]]:
+    def execute(self, status=None) -> List[Dict[str, Any]]:
+        status.update("griffoning: searching component-registry.")
         results = []
         params = {
-            "include_fields": "link,purl,type,name,related_url,namespace,software_build,nvr,sources.nvr,sources.purl,sources.name,sources.namespace,sources.download_url,sources.related_url,upstreams.nvr,upstreams.purl,upstreams.namespace,upstreams.name,upstreams.download_url,upstreams.related_url,release,version,arch,product_streams.product_versions,product_streams.name,product_streams.ofuri,product_streams.active,product_streams.exclude_components",  # noqa
+            "limit": 75,
+            "include_fields": "purl,type,name,related_url,namespace,software_build,nvr,release,version,arch,product_streams.product_versions,product_streams.name,product_streams.ofuri,product_streams.active,product_streams.exclude_components",  # noqa
         }
-
         if self.search_latest:
             params["latest_components_by_streams"] = "True"
             if not self.strict_name_search:
@@ -288,12 +292,36 @@ class products_containing_component_query:
                 params["name"] = self.component_name
             if self.ns:
                 params["namespace"] = self.ns
-
-            component_initial = self.corgi_session.components.retrieve_list(limit=120, **params)
-            latest_components: list = async_retrieve_components(
-                self.corgi_session, params, component_initial, component_initial.count
-            )
-            results.extend(latest_components)
+            status.update("griffoning: searching latest component(s).")
+            latest_components_cnt = self.corgi_session.components.count(**params)
+            status.update(f"griffoning: found {latest_components_cnt} latest component(s).")
+            latest_components = self.corgi_session.components.retrieve_list_iterator_async(**params)
+            with multiprocessing.Pool() as pool:
+                status.update(
+                    f"griffoning: found {latest_components_cnt} latest component(s), retrieving sources & upstreams."  # noqa
+                )
+                for processed_component in pool.map(
+                    partial(process_component, self.corgi_session), latest_components
+                ):
+                    results.append(processed_component)
+            if not self.no_community:
+                status.update("griffoning: searching latest community component(s).")
+                community_component_cnt = self.community_session.components.count(**params)
+                status.update(
+                    f"griffoning: found {community_component_cnt} latest community component(s)."  # noqa
+                )
+                latest_community_components = (
+                    self.community_session.components.retrieve_list_iterator_async(**params)
+                )
+                with multiprocessing.Pool() as pool:
+                    status.update(
+                        f"griffoning: found {community_component_cnt} latest community component(s), retrieving sources & upstreams."  # noqa
+                    )
+                    for processed_component in pool.map(
+                        partial(process_component, self.community_session),
+                        latest_community_components,
+                    ):
+                        results.append(processed_component)
 
         if self.search_related_url:
             # Note: related_url filter has no concept of strict
@@ -302,12 +330,41 @@ class products_containing_component_query:
                 params["namespace"] = self.ns
             if self.component_type:
                 params["type"] = self.component_type
-
-            component_initial = self.corgi_session.components.retrieve_list(limit=120, **params)
-            related_url_components: list = async_retrieve_components(
-                self.corgi_session, params, component_initial, component_initial.count
+            params["released_components"] = "True"
+            related_url_components_cnt = self.corgi_session.components.count(**params)
+            status.update(
+                f"griffoning: found {related_url_components_cnt} related url component(s)."
             )
-            results.extend(related_url_components)
+            related_url_components = self.corgi_session.components.retrieve_list_iterator_async(
+                **params
+            )
+            with multiprocessing.Pool() as pool:
+                status.update(
+                    f"griffoning: found {related_url_components_cnt} related url component(s), retrieving sources & upstreams."  # noqa
+                )
+                for processed_component in pool.map(
+                    partial(process_component, self.corgi_session), related_url_components
+                ):
+                    results.append(processed_component)
+            if not self.no_community:
+                latest_community_url_components_cnt = self.community_session.components.count(
+                    **params
+                )
+                status.update(
+                    f"griffoning: found {latest_community_url_components_cnt} related url community component(s)."  # noqa
+                )
+                latest_community_url_components = (
+                    self.community_session.components.retrieve_list_iterator_async(**params)
+                )
+                with multiprocessing.Pool() as pool:
+                    status.update(
+                        f"griffoning: found {latest_community_url_components_cnt} related url community component(s), retrieving sources & upstreams."  # noqa
+                    )
+                    for processed_component in pool.map(
+                        partial(process_component, self.community_session),
+                        latest_community_url_components,
+                    ):
+                        results.append(processed_component)
 
         if self.search_all:
             if not self.strict_name_search:
@@ -318,48 +375,70 @@ class products_containing_component_query:
                 params["type"] = self.component_type
             if self.ns:
                 params["namespace"] = self.ns
-
-            all_component_initial = self.corgi_session.components.retrieve_list(limit=120, **params)
-            all_components: list = async_retrieve_components(
-                self.corgi_session,
-                params,
-                all_component_initial,
-                all_component_initial.count,
+            params["released_components"] = "True"
+            all_components_cnt = self.corgi_session.components.count(**params)
+            status.update(f"griffoning: found {all_components_cnt} all component(s).")
+            # TODO: remove max_results
+            all_components = self.corgi_session.components.retrieve_list_iterator_async(
+                **params, max_results=10000
             )
-            results.extend(all_components)
+            status.update(f"griffoning: found {all_components_cnt} all component(s).")
+            with multiprocessing.Pool() as pool:
+                status.update(
+                    f"griffoning: found {all_components_cnt} all component(s), retrieving sources & upstreams."  # noqa
+                )
+                for processed_component in pool.map(
+                    partial(process_component, self.corgi_session), all_components
+                ):
+                    results.append(processed_component)
+
+            if not self.no_community:
+                all_community_components_cnt = self.community_session.components.count(**params)
+                status.update(
+                    f"griffoning: found {all_community_components_cnt} community all component(s)."  # noqa
+                )
+                # TODO: remove max_results
+                all_community_components = (
+                    self.community_session.components.retrieve_list_iterator_async(
+                        **params, max_results=10000
+                    )
+                )
+                with multiprocessing.Pool() as pool:
+                    status.update(
+                        f"griffoning: found {all_community_components_cnt} community all component(s), retrieving sources & upstreams."  # noqa
+                    )
+                    for processed_component in pool.map(
+                        partial(process_component, self.community_session),
+                        all_community_components,
+                    ):
+                        results.append(processed_component)
 
         if self.search_all_roots:
-            params["type"] = "RPM"
-            params["arch"] = "src"
+            params["root_components"] = "True"
             if not self.strict_name_search:
                 params["re_name"] = self.component_name
             else:
                 params["name"] = self.component_name
             if self.ns:
                 params["namespace"] = self.ns
-
-            all_src_component_initial = self.corgi_session.components.retrieve_list(
-                limit=120, **params
+            params["released_components"] = "True"
+            all_src_components_cnt = self.corgi_session.components.count(**params)
+            status.update(f"griffoning: found {all_src_components_cnt} all root component(s).")
+            all_src_components = self.corgi_session.components.retrieve_list_iterator_async(
+                **params
             )
-            all_src_components: list = async_retrieve_components(
-                self.corgi_session,
-                params,
-                all_src_component_initial,
-                all_src_component_initial.count,
-            )
-            params["type"] = "OCI"
-            params["arch"] = "noarch"
-            all_noarch_component_initial = self.corgi_session.components.retrieve_list(
-                limit=120, **params
-            )
-            all_noarch_components: list = async_retrieve_components(
-                self.corgi_session,
-                params,
-                all_noarch_component_initial,
-                all_noarch_component_initial.count,
-            )
-            all_root_components = all_src_components + all_noarch_components
-            results.extend(all_root_components)
+            for c in all_src_components:
+                results.append(c)
+            if not self.no_community:
+                all_src_community_components_cnt = self.community_session.components.count(**params)
+                all_src_community_components = (
+                    self.community_session.components.retrieve_list_iterator_async(**params)
+                )
+                status.update(
+                    f"griffoning: found {all_src_community_components_cnt} community all root component(s)."  # noqa
+                )
+                for c in all_src_community_components:
+                    results.append(c)
 
         if self.search_upstreams:
             # Note: upstreams only takes a purl ... so we must use re_upstreams for
@@ -371,12 +450,36 @@ class products_containing_component_query:
                 params["name"] = self.component_name
             if self.component_type:
                 params["type"] = self.component_type
-
-            component_initial = self.corgi_session.components.retrieve_list(limit=120, **params)
-            upstream_components: list = async_retrieve_components(
-                self.corgi_session, params, component_initial, component_initial.count
+            upstream_components_cnt = self.corgi_session.components.count(**params)
+            status.update(f"griffoning: found {upstream_components_cnt} upstream component(s).")
+            upstream_components = self.corgi_session.components.retrieve_list_iterator_async(
+                **params
             )
-            results.extend(upstream_components)
+            with multiprocessing.Pool() as pool:
+                status.update(f"griffoning: found {upstream_components_cnt} upstream component(s).")
+                for processed_component in pool.map(
+                    partial(process_component, self.corgi_session), upstream_components
+                ):
+                    results.append(processed_component)
+            if not self.no_community:
+                commmunity_upstream_components_cnt = self.community_session.components.count(
+                    **params
+                )
+                status.update(
+                    f"griffoning: found {commmunity_upstream_components_cnt} community upstream component(s)."  # noqa
+                )
+                commmunity_upstream_components = (
+                    self.community_session.components.retrieve_list_iterator_async(**params)
+                )
+                with multiprocessing.Pool() as pool:
+                    status.update(
+                        f"griffoning: found {commmunity_upstream_components_cnt} community upstream component(s), retrieving sources & upstreams."  # noqa
+                    )
+                    for processed_component in pool.map(
+                        partial(process_component, self.community_session),
+                        commmunity_upstream_components,
+                    ):
+                        results.append(processed_component)
 
         if self.filter_rh_naming:
             flags = re.IGNORECASE
@@ -417,41 +520,28 @@ class products_containing_component_query:
 
             results = filtered_results
 
-        if not self.no_community and (
-            self.search_community or self.search_all or self.search_all_roots
-        ):
-            self.community_session = CommunityComponentService.create_session()
-            params["type"] = "RPM"
-            params["arch"] = "src"
+        if self.search_community:
             if not self.strict_name_search:
                 params["re_name"] = self.component_name
             else:
                 params["name"] = self.component_name
-            if self.search_upstreams:
-                params["namespace"] = "UPSTREAM"
             if self.ns:
                 params["namespace"] = self.ns
-
-            if self.component_type:
-                params["type"] = self.component_type
-
-            component_initial = self.community_session.components.retrieve_list(limit=120, **params)
-            commmunity_src_components: list = async_retrieve_components(
-                self.community_session, params, component_initial, component_initial.count
+            all_community_components_cnt = self.community_session.components.count(**params)
+            status.update(
+                f"griffoning: found {all_community_components_cnt} community all component(s)."  # noqa
             )
-            params["type"] = "OCI"
-            params["arch"] = "noarch"
-            component_initial_noarch = self.community_session.components.retrieve_list(
-                limit=120, **params
+            all_community_components = (
+                self.community_session.components.retrieve_list_iterator_async(**params)
             )
-            commmunity_noarch_components: list = async_retrieve_components(
-                self.community_session,
-                params,
-                component_initial_noarch,
-                component_initial_noarch.count,
-            )
-            community_components = commmunity_src_components + commmunity_noarch_components
-            results.extend(community_components)
+            with multiprocessing.Pool() as pool:
+                status.update(
+                    f"griffoning: found {all_community_components_cnt} community all component(s), retrieving sources & upstreams."  # noqa
+                )
+                for processed_component in pool.map(
+                    partial(process_component, self.community_session), all_community_components
+                ):
+                    results.append(processed_component)
 
         return results
 
@@ -478,7 +568,7 @@ class components_containing_specific_component_query:
         self.params = params
         self.purl = self.params.get("purl")
 
-    def execute(self) -> dict:
+    def execute(self, status=None) -> dict:
         if self.purl:
             c = self.corgi_session.components.retrieve_list(
                 purl=self.purl,
@@ -523,7 +613,7 @@ class components_containing_component_query:
         self.namespace = self.params.get("namespace")
         self.strict_name_search = self.params.get("strict_name_search")
 
-    def execute(self) -> List[Dict[str, Any]]:
+    def execute(self, status=None) -> List[Dict[str, Any]]:
         cond = {}
         if not self.strict_name_search:
             cond["re_name"] = self.component_name
@@ -536,29 +626,7 @@ class components_containing_component_query:
         if self.namespace:
             cond["namespace"] = self.namespace
 
-        components: List[Any] = []
-        logger.debug("starting parallel http requests")
-        component_cnt = self.corgi_session.components.retrieve_list(**cond).count
-        if component_cnt < 3000000:
-            with concurrent.futures.ThreadPoolExecutor() as executor:
-                futures = []
-                components = list()
-                for batch in range(0, component_cnt, 120):
-                    futures.append(
-                        executor.submit(
-                            self.corgi_session.components.retrieve_list,
-                            **cond,
-                            offset=batch,
-                            include_fields="link,name,type,arch,version,purl,nvr,sources,related_url,download_url",  # noqa
-                            limit=120,
-                        )
-                    )
-                for future in concurrent.futures.as_completed(futures):
-                    try:
-                        components.extend(future.result().results)
-                    except Exception as exc:
-                        logger.warning("%r generated an exception: %s" % (future, exc))
-
+        components = self.corgi_session.components.retrieve_list_iterator_async(**cond)
         results = []
         for c in components:
             sources = [{"link": source["link"], "purl": source["purl"]} for source in c.sources]
@@ -608,7 +676,7 @@ class components_affected_by_specific_cve_query:
         self.component_type = self.params.get("component_type")
         self.namespace = self.params.get("namespace")
 
-    def execute(self) -> dict:
+    def execute(self, status=None) -> dict:
         cond = {}
         if self.affectedness:
             cond["affectedness"] = self.affectedness
@@ -626,24 +694,15 @@ class components_affected_by_specific_cve_query:
         flaw = self.osidb_session.flaws.retrieve(self.cve_id)
         affects = flaw.affects
         results = list()
-        with concurrent.futures.ThreadPoolExecutor() as executor:
-            futures = []
-            for affect in affects:
-                futures.append(
-                    executor.submit(
-                        self.corgi_session.components.retrieve_list,
-                        name=affect.ps_component,
-                        latest_components_by_streams=True,
-                        include_fields="purl,product_streams,product_versions,software_build",
-                    )
-                )
-            for future in concurrent.futures.as_completed(futures):
-                try:
-                    for c in future.result().results:
-                        results.append(c.to_dict())
-                except Exception as exc:
-                    logger.error("%r generated an exception: %s" % (future, exc))
-                    exit(0)
+
+        for affect in affects:
+            components = self.corgi_session.components.retrieve_list_iterator_async(
+                name=affect.ps_component,
+                latest_components_by_streams="True",
+                include_fields="purl,product_streams,product_versions,software_build",
+            )
+            for c in components:
+                results.append(c.to_dict())
 
         return {
             "link": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",
@@ -685,12 +744,12 @@ class cves_for_specific_component_query:
         self.affect_resolution = self.params.get("affect_resolution")
         self.affect_impact = self.params.get("affect_impact")
 
-    def execute(self) -> List[Dict[str, Any]]:
+    def execute(self, status=None) -> List[Dict[str, Any]]:
         components = []
         if self.component_name:
             affects: list = []
             params = {
-                "include_fields": "cve_id,title,state,resolution,impact,affects",
+                "include_fields": "cve_id,title,resolution,impact,affects",
             }
             params["affects__ps_component"] = self.component_name
             if self.flaw_state:
@@ -706,63 +765,42 @@ class cves_for_specific_component_query:
             if self.affect_impact:
                 params["affects__impact"] = self.affect_impact
 
-            res = requests.get(f"{OSIDB_API_URL}/osidb/api/v1/flaws", params=params)
-            flaws = res.json()
-            flaws_cnt = int(flaws["count"])
-            with concurrent.futures.ThreadPoolExecutor() as executor:
-                futures = []
-                flaws = list()
-                for batch in range(0, flaws_cnt, 75):
-                    params["offset"] = batch  # type: ignore
-                    params["limit"] = 75  # type: ignore
-                    futures.append(
-                        executor.submit(
-                            requests.get,
-                            f"{OSIDB_API_URL}/osidb/api/v1/flaws",
-                            params=params,
-                        )
-                    )
-                for future in concurrent.futures.as_completed(futures):
-                    try:
-                        flaws.extend(future.result().json()["results"])
-                    except Exception as exc:
-                        logger.warning("%r generated an exception: %s" % (future, exc))
+            flaws = self.osidb_session.flaws.retrieve_list_iterator_async(**params)
 
-                for flaw in flaws:
-                    for affect in flaw["affects"]:
-                        if self.affectedness:
-                            if self.affectedness != affect["affectedness"]:
-                                continue
-                        if self.affect_resolution:
-                            if self.affect_resolution != affect["resolution"]:
-                                continue
-                        if self.affect_impact:
-                            if self.affect_impact != affect["impact"]:
-                                continue
-                        affects.append(
-                            {
-                                "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect['uuid']}",  # noqa
-                                "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw['cve_id']}",  # noqa
-                                "link_component": f"{CORGI_API_URL}/api/v1/components?name={affect['ps_component']}&latest_components_by_streams=True",  # noqa
-                                "link_community_component": f"{COMMUNITY_COMPONENTS_API_URL}/api/v1/components?name={affect['ps_component']}&latest_components_by_streams=True",  # noqa
-                                "flaw_cve_id": flaw["cve_id"],
-                                "title": flaw["title"],
-                                "flaw_state": flaw["state"],
-                                "flaw_resolution": flaw["resolution"],
-                                "affect_component_name": affect["ps_component"],
-                                "affect_product_version": affect["ps_module"],
-                                "affect_affectedness": affect["affectedness"],
-                                "affect_impact": affect["impact"],
-                                "affect_resolution": affect["resolution"],
-                            }
-                        )
-                components.append(
-                    {
-                        "link": f"{CORGI_API_URL}/api/v1/components?name={self.component_name}",
-                        "name": self.component_name,
-                        "affects": affects,
-                    }
-                )
+            for flaw in flaws:
+                for affect in flaw.affects:
+                    if self.affectedness:
+                        if self.affectedness != affect.affectedness:
+                            continue
+                    if self.affect_resolution:
+                        if self.affect_resolution != affect.resolution:
+                            continue
+                    if self.affect_impact:
+                        if self.affect_impact != affect.impact:
+                            continue
+                    affects.append(
+                        {
+                            "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect.uuid}",  # noqa
+                            "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",  # noqa
+                            "link_component": f"{CORGI_API_URL}/api/v1/components?name={affect.ps_component}&latest_components_by_streams=True",  # noqa
+                            "link_community_component": f"{COMMUNITY_COMPONENTS_API_URL}/api/v1/components?name={affect.ps_component}&latest_components_by_streams=True",  # noqa
+                            "flaw_cve_id": flaw.cve_id,
+                            "title": flaw.title,
+                            "flaw_resolution": flaw.resolution,
+                            "affect_component_name": affect.ps_component,
+                            "affect_product_version": affect.ps_module,
+                            "affect_affectedness": affect.affectedness,
+                            "affect_impact": affect.impact,
+                            "affect_resolution": affect.resolution,
+                        }
+                    )
+            components.append(
+                {
+                    "link": f"{CORGI_API_URL}/api/v1/components?name={self.component_name}",
+                    "name": self.component_name,
+                    "affects": affects,
+                }
+            )
 
         if self.purl:
             pass
@@ -798,16 +836,14 @@ class cves_for_specific_product_query:
         self.affect_resolution = self.params.get("affect_resolution")
         self.affect_impact = self.params.get("affect_impact")
 
-    def execute(self) -> List[Dict[str, Any]]:
+    def execute(self, status=None) -> List[Dict[str, Any]]:
         components = []
         if self.product_version_name:
             affects: list = []
             params = {
-                "include_fields": "cve_id,title,state,resolution,impact,affects",
+                "include_fields": "cve_id,title,resolution,impact,affects",
             }
             params["affects__ps_module"] = self.product_version_name
-            if self.flaw_state:
-                params["state"] = self.flaw_state
             if self.flaw_resolution:
                 params["resolution"] = self.flaw_resolution
             if self.flaw_impact:
@@ -819,62 +855,41 @@ class cves_for_specific_product_query:
             if self.affect_impact:
                 params["affects__impact"] = self.affect_impact
 
-            res = requests.get(f"{OSIDB_API_URL}/osidb/api/v1/flaws", params=params)
-            flaws = res.json()
-            flaws_cnt = int(flaws["count"])
-            with concurrent.futures.ThreadPoolExecutor() as executor:
-                futures = []
-                flaws = list()
-                for batch in range(0, flaws_cnt, 75):
-                    params["offset"] = batch  # type: ignore
-                    params["limit"] = 75  # type: ignore
-                    futures.append(
-                        executor.submit(
-                            requests.get,
-                            f"{OSIDB_API_URL}/osidb/api/v1/flaws",
-                            params=params,
-                        )
+            flaws = self.osidb_session.flaws.retrieve_list_iterator_async(**params)
+            for flaw in flaws:
+                for affect in flaw.affects:
+                    if self.affectedness:
+                        if self.affectedness != affect.affectedness:
+                            continue
+                    if self.affect_resolution:
+                        if self.affect_resolution != affect.resolution:
+                            continue
+                    if self.affect_impact:
+                        if self.affect_impact != affect.impact:
+                            continue
+                    affects.append(
+                        {
+                            "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect.uuid}",  # noqa
+                            "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",  # noqa
+                            "link_component": f"{CORGI_API_URL}/api/v1/components?name={affect.ps_component}&latest_components_by_streams=True",  # noqa
+                            "link_community_component": f"{COMMUNITY_COMPONENTS_API_URL}/api/v1/components?name={affect.ps_component}&latest_components_by_streams=True",  # noqa
+                            "flaw_cve_id": flaw.cve_id,
+                            "title": flaw.title,
+                            "flaw_state": flaw.state,
+                            "flaw_resolution": flaw.resolution,
+                            "affect_component_name": affect.ps_component,
+                            "affect_product_version": affect.ps_module,
+                            "affect_affectedness": affect.affectedness,
+                            "affect_impact": affect.impact,
+                            "affect_resolution": affect.resolution,
+                        }
                     )
-                for future in concurrent.futures.as_completed(futures):
-                    try:
-                        flaws.extend(future.result().json()["results"])
-                    except Exception as exc:
-                        logger.warning("%r generated an exception: %s" % (future, exc))
-
-                for flaw in flaws:
-                    for affect in flaw["affects"]:
-                        if self.affectedness:
-                            if self.affectedness != affect["affectedness"]:
-                                continue
-                        if self.affect_resolution:
-                            if self.affect_resolution != affect["resolution"]:
-                                continue
-                        if self.affect_impact:
-                            if self.affect_impact != affect["impact"]:
-                                continue
-                        affects.append(
-                            {
-                                "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect['uuid']}",  # noqa
-                                "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw['cve_id']}",  # noqa
-                                "link_component": f"{CORGI_API_URL}/api/v1/components?name={affect['ps_component']}&latest_components_by_streams=True",  # noqa
-                                "link_community_component": f"{COMMUNITY_COMPONENTS_API_URL}/api/v1/components?name={affect['ps_component']}&latest_components_by_streams=True",  # noqa
-                                "flaw_cve_id": flaw["cve_id"],
-                                "title": flaw["title"],
-                                "flaw_state": flaw["state"],
-                                "flaw_resolution": flaw["resolution"],
-                                "affect_component_name": affect["ps_component"],
-                                "affect_product_version": affect["ps_module"],
-                                "affect_affectedness": affect["affectedness"],
-                                "affect_impact": affect["impact"],
-                                "affect_resolution": affect["resolution"],
-                            }
-                        )
-                components.append(
-                    {
-                        "link": f"{CORGI_API_URL}/api/v1/product_versions?name={self.product_version_name}",  # noqa
-                        "name": self.product_version_name,
-                        "affects": affects,
-                    }
-                )
+            components.append(
+                {
+                    "link": f"{CORGI_API_URL}/api/v1/product_versions?name={self.product_version_name}",  # noqa
+                    "name": self.product_version_name,
+                    "affects": affects,
+                }
+            )
 
         return components
